%%p = ellipseConic2param(C) 
% estimates values of parametric represenation of a 2D ellipse
% C: double 3x3 ellipse-conic, det(Chh)!>0
% p: double 5x1, [x0,y0,a,b,phi]
%
% Susanne Wenzel 09/16
% wenzel@igg.uni-bonn.de
function p = ellipseConic2param(C)

% help values
Chh = C(1:2,1:2);

if det(Chh)<=0
    error('C does not fullfill requirements to be an ellipse, det(Chh)!>0')
end

ch0 = C(1:2,  3);
c00 = C(  3,  3);

% centre point
x0 = -inv(Chh+eps*eye(2)) *ch0; 

% eigenvalue decomposition of normalized homogeneous part of the conic
c00q = c00 - ch0'*inv(Chh+eps*eye(2))*ch0;
% eigenvalues
[R,Lambda] = eig( - Chh/ (c00q+eps) );
lambda = diag(Lambda);
if lambda(1)<0
    lambda = -lambda;
end
% semi axes
% eigenvalues are not sorted obligatory
[minlambda,idx_min] = min(lambda);
[maxlambda,idx_max] = max(lambda);

% semi axes
a= 1/sqrt(lambda(idx_min));
b= 1/sqrt(lambda(idx_max));


if  abs(minlambda-maxlambda)<eps
    % its a circle, orientation 0
    phi = 0;
else
    phi = atan2(R(2,idx_min),R(1,idx_min));
    if phi<0
        phi = phi+2*pi;
    end
end

 p = [x0',a,b,phi];